//
//  ARSCNViewController.m
//  ARDemo3
//
//  Created by 汇元 on 2017/12/18.
//  Copyright © 2017年 汇元. All rights reserved.
//

#import "ARSCNViewController.h"

// 3d游戏框架
#import <SceneKit/SceneKit.h>
// ARKit框架
#import <ARKit/ARKit.h>

@interface ARSCNViewController ()

// 全局属性
//AR视图：展示3D界面
@property(nonatomic,strong) ARSCNView * arSCNView;

//AR会话，负责管理相机追踪配置及3D相机坐标
@property(nonatomic,strong) ARSession * arSession;

//会话追踪配置：负责追踪相机的运动
@property(nonatomic,strong) ARConfiguration * arConfiguration;

//飞机3D模型(本小节加载多个模型)
@property(nonatomic,strong) SCNNode * planeNode;

@end

@implementation ARSCNViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor whiteColor];
    // Do any additional setup after loading the view.
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    
    //1.将AR视图添加到当前视图
    [self.view addSubview:self.arSCNView];
    //2.开启AR会话（此时相机开始工作）
    [self.arSession runWithConfiguration:self.arConfiguration];
}

- (void)touchesEnded:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    //1.使用场景加载scn文件（scn格式文件是一个基于3D建模的文件，使用3DMax软件可以创建，这里系统有一个默认的3D飞机）--------在右侧我添加了许多3D模型，只需要替换文件名即可
    SCNScene *scene = [SCNScene sceneNamed:@"art.scnassets/ship.scn"];
    //2.获取飞机节点（一个场景会有多个节点，此处我们只写，飞机节点则默认是场景子节点的第一个）
    //所有的场景有且只有一个根节点，其他所有节点都是根节点的子节点
    SCNNode *shipNode = scene.rootNode.childNodes[0];
    
    //3.将飞机节点添加到当前屏幕中
    [self.arSCNView.scene.rootNode addChildNode:shipNode];
}

#pragma mark - 懒加载
#pragma mark - 搭建ARKit环境
// 懒加载会话追踪配置
- (ARConfiguration *)arConfiguration {
    if (_arConfiguration != nil) {
        return _arConfiguration;
    }
    //1.创建世界追踪会话配置（使用ARWorldTrackingSessionConfiguration效果更加好），需要A9芯片支持
    ARWorldTrackingConfiguration *configuration = [[ARWorldTrackingConfiguration alloc] init];
    //2.设置追踪方向（追踪平面，后面会用到）
    configuration.planeDetection = ARPlaneDetectionHorizontal;
    _arConfiguration = configuration;
    //3.自适应灯光（相机从暗到强光快速过渡效果会平缓一些）
    _arConfiguration.lightEstimationEnabled = YES;
    
    return _arConfiguration;
}

// 懒加载拍摄会话
- (ARSession *)arSession {
    if (_arSession != nil) {
        return _arSession;
    }
    //1.创建会话
    _arSession = [[ARSession alloc] init];
    //2返回会话
    return _arSession;
}

//创建AR视图
- (ARSCNView *)arSCNView {
    if (_arSCNView != nil) {
        return _arSCNView;
    }
    //1.创建AR视图
    _arSCNView = [[ARSCNView alloc] initWithFrame:self.view.bounds];
    //2.设置视图会话
    _arSCNView.session = self.arSession;
    //3.自动刷新灯光（3D游戏用到，此处可忽略）
    _arSCNView.automaticallyUpdatesLighting = YES;
    
    return _arSCNView;
}

@end
